@@ -10,7 +10,7 @@
 ;; Created: 2008-05-13
 ;; Keywords: convenience
 ;; EmacsWiki: IdleHighlight
-;; Package-Requires: ((emacs "27.1"))
+;; Package-Requires: ((emacs "29.1"))
 
 ;;; Commentary:
 
@@ -54,16 +54,14 @@
 (defcustom idle-highlight-exceptions nil
   "List of words to be excepted from highlighting."
   :type
-  '(choice
-    (repeat :tag "A list of string literals that will be excluded." string)
-    (function :tag "A function taking a string, non-nil result excludes.")))
+  '(choice (repeat :tag "A list of string literals that will be excluded." string)
+           (function :tag "A function taking a string, non-nil result excludes.")))
 
 (defcustom idle-highlight-exceptions-face '(font-lock-keyword-face font-lock-string-face)
   "List of exception faces."
   :type
-  '(choice
-    (repeat :tag "A list of face symbols that will be ignored." symbol)
-    (function :tag "A function that takes a list of faces, non-nil result excludes.")))
+  '(choice (repeat :tag "A list of face symbols that will be ignored." symbol)
+           (function :tag "A function that takes a list of faces, non-nil result excludes.")))
 
 (defcustom idle-highlight-exceptions-syntax "^w_"
   "Syntax table to to skip.
@@ -75,6 +73,10 @@ See documentation for `skip-syntax-forward', nil to ignore."
   "Exclude the current symbol from highlighting."
   :type 'boolean)
 
+(defcustom idle-highlight-before-point nil
+  "Highlight the text directly before the cursor."
+  :type 'boolean)
+
 (defcustom idle-highlight-visible-buffers nil
   "Apply the current highlight to all other visible buffers."
   :type 'boolean)
@@ -88,9 +90,7 @@ See documentation for `skip-syntax-forward', nil to ignore."
   :type '(repeat symbol))
 
 (define-obsolete-variable-alias
-  'global-idle-highlight-ignore-buffer
-  'idle-highlight-global-ignore-buffer
-  "1.1.4")
+  'global-idle-highlight-ignore-buffer 'idle-highlight-global-ignore-buffer "1.1.4")
 
 (defvar-local idle-highlight-global-ignore-buffer nil
   "When non-nil, the global mode will not be enabled for this buffer.
@@ -112,6 +112,7 @@ check this buffer.")
 (defun idle-highlight--faces-at-point (pos)
   "Add the named faces that the `read-face-name' or `face' property use.
 Argument POS return faces at this point."
+  (declare (important-return-value t))
   (let ((faces nil) ; List of faces to return.
         ;; NOTE: use `get-text-property' instead of `get-char-property' so overlays are excluded,
         ;; since this causes overlays with `hl-line-mode' (for example) to mask keywords, see: #1.
@@ -129,13 +130,14 @@ Argument POS return faces at this point."
   "Destructively modify and return RANGES with overlapping values removed.
 
 Where RANGES is an unordered list of (min . max) cons cells."
+  (declare (important-return-value t))
   (cond
    ((cdr ranges)
     ;; Simple < sorting of cons cells.
     (setq ranges
-          (sort
-           ranges
-           (lambda (x y) (or (< (car x) (car y)) (and (= (car x) (car y)) (< (cdr x) (cdr y)))))))
+          (sort ranges
+                (lambda (x y)
+                  (or (< (car x) (car y)) (and (= (car x) (car y)) (< (cdr x) (cdr y)))))))
     ;; Step over `ranges', de-duplicating & adjusting elements as needed.
     (let ((ranges-iter ranges)
           (ranges-next (cdr ranges)))
@@ -162,14 +164,27 @@ Where RANGES is an unordered list of (min . max) cons cells."
 
 (defun idle-highlight--check-symbol-at-point (pos)
   "Return non-nil if the symbol at POS can be used."
+  (declare (important-return-value t))
   (cond
    (idle-highlight-exceptions-syntax
-    (save-excursion (zerop (skip-syntax-forward idle-highlight-exceptions-syntax (1+ pos)))))
+    (save-excursion
+      (cond
+       (idle-highlight-before-point
+        (or (progn
+              (goto-char pos)
+              (zerop (skip-syntax-forward idle-highlight-exceptions-syntax (1+ pos))))
+            (progn
+              (goto-char pos)
+              (zerop (skip-syntax-backward idle-highlight-exceptions-syntax pos)))))
+       (t
+        (goto-char pos)
+        (zerop (skip-syntax-forward idle-highlight-exceptions-syntax (1+ pos)))))))
    (t
     t)))
 
 (defun idle-highlight--check-faces-at-point (pos)
   "Check if the position POS has faces that match the exclude argument."
+  (declare (important-return-value t))
   (cond
    (idle-highlight-exceptions-face
     (let ((result t))
@@ -192,6 +207,7 @@ Where RANGES is an unordered list of (min . max) cons cells."
 
 (defun idle-highlight--check-word (target)
   "Return non-nil when TARGET should not be excluded."
+  (declare (important-return-value t))
   (not
    (cond
     ((functionp idle-highlight-exceptions)
@@ -205,6 +221,7 @@ Where RANGES is an unordered list of (min . max) cons cells."
 
 (defun idle-highlight--unhighlight ()
   "Clear current highlight."
+  (declare (important-return-value nil))
   (when idle-highlight--overlays
     (mapc #'delete-overlay idle-highlight--overlays)
     (setq idle-highlight--overlays nil)))
@@ -213,23 +230,26 @@ Where RANGES is an unordered list of (min . max) cons cells."
   "Highlight TARGET found between TARGET-BEG and TARGET-END.
 
 Argument VISIBLE-RANGES is a list of (min . max) ranges to highlight."
+  (declare (important-return-value nil))
   (idle-highlight--unhighlight)
   (save-excursion
-    (let ((target-regexp (concat "\\_<" (regexp-quote target) "\\_>")))
-      (pcase-dolist (`(,beg . ,end) visible-ranges)
-        (goto-char beg)
-        (while (re-search-forward target-regexp end t)
-          (let ((match-beg (match-beginning 0))
-                (match-end (match-end 0)))
-            (unless (and idle-highlight-exclude-point
-                         (eq target-beg match-beg)
-                         (eq target-end match-end))
-              (let ((ov (make-overlay match-beg match-end)))
-                (overlay-put ov 'face 'idle-highlight)
-                (push ov idle-highlight--overlays)))))))))
+    (save-match-data
+      (let ((target-regexp (concat "\\_<" (regexp-quote target) "\\_>")))
+        (pcase-dolist (`(,beg . ,end) visible-ranges)
+          (goto-char beg)
+          (while (re-search-forward target-regexp end t)
+            (let ((match-beg (match-beginning 0))
+                  (match-end (match-end 0)))
+              (unless (and idle-highlight-exclude-point
+                           (eq target-beg match-beg)
+                           (eq target-end match-end))
+                (let ((ov (make-overlay match-beg match-end)))
+                  (overlay-put ov 'face 'idle-highlight)
+                  (push ov idle-highlight--overlays))))))))))
 
 (defun idle-highlight--word-at-point-args ()
   "Return arguments for `idle-highlight--highlight'."
+  (declare (important-return-value t))
   (when (idle-highlight--check-symbol-at-point (point))
     (let ((target-range (bounds-of-thing-at-point 'symbol)))
       (when (and target-range (idle-highlight--check-faces-at-point (point)))
@@ -243,6 +263,7 @@ Argument VISIBLE-RANGES is a list of (min . max) ranges to highlight."
 
 Arguments TARGET and TARGET-RANGE
 should be the result of `idle-highlight--word-at-point-args'."
+  (declare (important-return-value nil))
   (idle-highlight--unhighlight)
   (when target
     (pcase-let ((`(,target-beg . ,target-end) target-range))
@@ -275,6 +296,7 @@ should be the result of `idle-highlight--word-at-point-args'."
 
 (defun idle-highlight--time-callback-or-disable ()
   "Callback that run the repeat timer."
+  (declare (important-return-value nil))
 
   ;; Ensure all other buffers are highlighted on request.
   (let ((is-mode-active (bound-and-true-p idle-highlight-mode))
@@ -318,10 +340,10 @@ should be the result of `idle-highlight--word-at-point-args'."
                 (push (cons
                        (progn
                          (goto-char (max (point-min) (window-start win)))
-                         (line-beginning-position))
+                         (pos-bol))
                        (progn
                          (goto-char (min (point-max) (window-end win)))
-                         (line-end-position)))
+                         (pos-eol)))
                       idle-highlight--dirty)))))))
 
 
@@ -352,6 +374,7 @@ should be the result of `idle-highlight--word-at-point-args'."
 
 (defun idle-highlight--time-ensure (state)
   "Ensure the timer is enabled when STATE is non-nil, otherwise disable."
+  (declare (important-return-value nil))
   (cond
    (state
     (unless idle-highlight--global-timer
@@ -366,6 +389,7 @@ should be the result of `idle-highlight--word-at-point-args'."
 
 (defun idle-highlight--time-reset ()
   "Run this when the buffer change was changed."
+  (declare (important-return-value nil))
   ;; Ensure changing windows doesn't leave other buffers with stale highlight.
   (cond
    ((bound-and-true-p idle-highlight-mode)
@@ -377,6 +401,7 @@ should be the result of `idle-highlight--word-at-point-args'."
 
 (defun idle-highlight--time-buffer-local-enable ()
   "Ensure buffer local state is enabled."
+  (declare (important-return-value nil))
   ;; Needed in case focus changes before the idle timer runs.
   (setq idle-highlight--dirty-flush-all t)
   (setq idle-highlight--dirty t)
@@ -385,6 +410,7 @@ should be the result of `idle-highlight--word-at-point-args'."
 
 (defun idle-highlight--time-buffer-local-disable ()
   "Ensure buffer local state is disabled."
+  (declare (important-return-value nil))
   (kill-local-variable 'idle-highlight--dirty)
   (idle-highlight--time-ensure nil)
   (remove-hook 'window-state-change-hook #'idle-highlight--time-reset t))
@@ -395,16 +421,19 @@ should be the result of `idle-highlight--word-at-point-args'."
 
 (defun idle-highlight--enable ()
   "Enable the buffer local minor mode."
+  (declare (important-return-value nil))
   (idle-highlight--time-buffer-local-enable))
 
 (defun idle-highlight--disable ()
   "Disable the buffer local minor mode."
+  (declare (important-return-value nil))
   (idle-highlight--time-buffer-local-disable)
   (idle-highlight--unhighlight)
   (kill-local-variable 'idle-highlight--overlays))
 
 (defun idle-highlight--turn-on ()
   "Enable command `idle-highlight-mode'."
+  (declare (important-return-value nil))
   (when (and
          ;; Not already enabled.
          (not (bound-and-true-p idle-highlight-mode))
@@ -450,5 +479,6 @@ should be the result of `idle-highlight--word-at-point-args'."
 ;; Local Variables:
 ;; fill-column: 99
 ;; indent-tabs-mode: nil
+;; elisp-autofmt-format-quoted: nil
 ;; End:
 ;;; idle-highlight-mode.el ends here
